@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 import logging
-import traceback
 import typing as t
 from contextlib import AsyncExitStack
 
@@ -10,8 +11,10 @@ from starlette.types import ASGIApp, Receive, Scope, Send
 from kupala.container import Container
 
 from .config import Config
+from .contracts import Debug
 from .dotenv import Env
-from .extensions import Extensions
+from .errors import ErrorHandlers, ExceptionMiddleware
+from .extensions import Extension, Extensions
 from .middleware import MiddlewareStack
 from .requests import Request
 from .routing import Router, Routes
@@ -23,68 +26,60 @@ class App(Container):
         config: dict = None,
         debug: bool = False,
         env_prefix: str = "",
+        extensions: list[Extension] = None,
     ) -> None:
         super().__init__()
         self.commands: list[click.Command] = []
         self.config = Config(config)
         self.debug = debug
         self.env = Env(env_prefix)
-        self.extensions = Extensions()
+        self.extensions = Extensions(extensions or [])
+        self.error_handlers = ErrorHandlers()
         self.lifecycle: list[t.Callable[[App], t.AsyncContextManager]] = []
         self.middleware = MiddlewareStack()
         self.routes = Routes()
 
-        self._router: t.Optional[Router] = None
         self._asgi_app_instance: t.Optional[ASGIApp] = None
         self._booted = False
 
+        self.bind(Debug, self.debug, aliases="debug")
+        self.bind(App, self, "app")
+        self.bind(Env, self.env, "env")
         self.bind(Config, self.config, aliases="config")
-        self.bind("debug", self.debug)
+        self.bind(Routes, self.routes, aliases="routes")
+        self.bind(MiddlewareStack, self.middleware, aliases="middleware")
+        self.bind(Extensions, self.extensions, aliases="extensions")
 
     @property
     def asgi_app(self) -> ASGIApp:
         """Create ASGI application. Once created a cached instance returned."""
-        if self._asgi_app_instance is None:
-            self._bootstrap()
-            self._router = Router(self.routes)
-            app: ASGIApp = self._router
-            self.middleware.use(ServerErrorMiddleware, debug=self.debug)
-            for mw in reversed(self.middleware):
-                app = mw.wrap(app)
-            self._asgi_app_instance = app
-        return self._asgi_app_instance
+        try:
+            if self._asgi_app_instance is None:
+                self.bootstrap()
+                app: ASGIApp = self.get(Router)
+                self.middleware.top(
+                    ExceptionMiddleware,
+                    handlers=self.error_handlers,
+                )
+                self.middleware.top(ServerErrorMiddleware, debug=self.debug)
+                for mw in reversed(self.middleware):
+                    app = mw.wrap(app)
+                self._asgi_app_instance = app
+        except Exception as ex:
+            logging.exception(ex)
+            raise
+        else:
+            return self._asgi_app_instance
 
     def run_cli(self) -> None:
         """Run command line application."""
-        self._bootstrap()
+        self.bootstrap()
         app = click.Group()
         for command in self.commands:
             app.add_command(command)
         app()
 
-    async def lifespan(
-        self,
-        scope: Scope,
-        receive: Receive,
-        send: Send,
-    ) -> None:
-        """ASGI lifespan events handler."""
-        try:
-            await receive()
-            async with AsyncExitStack() as stack:
-                for hook in self.lifecycle:
-                    await stack.enter_async_context(hook(self))
-                await send({"type": "lifespan.startup.complete"})
-                await receive()
-        except BaseException as ex:
-            logging.exception(ex)
-            text = traceback.format_exc()
-            await send({"type": "lifespan.startup.failed", "message": text})
-            raise
-        else:
-            await send({"type": "lifespan.shutdown.complete"})
-
-    def _bootstrap(self) -> None:
+    def bootstrap(self) -> None:
         if self._booted:
             return
 
@@ -96,6 +91,12 @@ class App(Container):
         for ext in self.extensions:
             ext.bootstrap(self)
 
+    async def lifespan(self, app: App) -> t.AsyncGenerator:
+        async with AsyncExitStack() as stack:
+            for hook in self.lifecycle:
+                await stack.enter_async_context(hook(app))
+            yield
+
     async def __call__(
         self,
         scope: Scope,
@@ -105,10 +106,6 @@ class App(Container):
         """ASGI entry point."""
         assert scope["type"] in {"http", "websocket", "lifespan"}
 
-        if scope["type"] == "lifespan":
-            await self.lifespan(scope, receive, send)
-            return
-
         scope["app"] = self
         if scope["type"] == "http":
             scope["request"] = Request(scope, receive, send)
