@@ -1,7 +1,12 @@
 import abc
 import typing as t
 
+from kupala.requests import Request
 
+Debug = bool
+
+
+@t.runtime_checkable
 class Identity(t.Protocol):  # pragma: no cover
     def get_identity(self) -> str:
         ...
@@ -14,13 +19,13 @@ class Identity(t.Protocol):  # pragma: no cover
 
 
 class IdentityProvider(abc.ABC):  # pragma: no cover
-    async def find_by_identity(self, identity: str) -> Identity:
+    async def find_by_identity(self, identity: str) -> t.Optional[Identity]:
         raise NotImplementedError()
 
-    async def find_by_id(self, id: t.Any) -> Identity:
+    async def find_by_id(self, id: t.Any) -> t.Optional[Identity]:
         raise NotImplementedError()
 
-    async def find_by_token(self, token: str) -> Identity:
+    async def find_by_token(self, token: str) -> t.Optional[Identity]:
         raise NotImplementedError()
 
 
@@ -41,3 +46,19 @@ class URLResolver(abc.ABC):  # pragma: no cover
         **path_params: dict,
     ) -> str:  # pragma: nocover
         raise NotImplementedError()
+
+
+class Authenticator(t.Protocol):
+    def __init__(self, user_provider: IdentityProvider, **kwargs: t.Any):
+        ...
+
+    async def authenticate(self, request: Request) -> t.Optional[Identity]:
+        ...
+
+
+class PasswordHasher(t.Protocol):
+    def hash(self, plain: str) -> str:
+        ...
+
+    def verify(self, plain: str, hashed: str) -> bool:
+        ...
